import java.util.Comparator;

public class Person implements Comparable {

	protected String name;
	protected int age;
	protected double weight;
	protected double height;
	protected Gender gender;
	
	public Person() {
		this("", 0, 0, 0, Gender.FEMALE);
	}
	
	public Person(String name, int age, double weight, double height, Gender gender) {
		this.name = name;
		this.age = age;
		this.weight = weight;
		this.height = height;
		this.gender = gender;
	}
	
	public Person(Person another) {
		this.name = another.name;
		this.age = another.age;
		this.weight = another.weight;
		this.height = another.height;
		this.gender = another.gender;
	}
	
	public void setName(String name) {
		this.name = name;
	}
	
	public void setAge(int age) {
		this.age = age;
	}
	
	public void setWeight(double weight) {
		this.weight = weight;
	}
	
	public void setHeight(double height) {
		this.height = height;
	}
	
	public void setGender(Gender gender) {
		this.gender = gender;
	}
	
	public String getName() {
		return name;
	}
	
	public int getAge() {
		return age;
	}
	
	public double getWeight() {
		return weight;
	}
	
	public double getHeight() {
		return height;
	}
	
	public Gender getGender() {
		return gender;
	}
	
	@Override
	public String toString() {
		return name +  ", " + age + ", " + weight + ", " + height + ", " + gender;
	}

	@Override
	public int compareTo(Object o) {
		Person other = (Person)o;
		return name.compareTo(other.name);
	}

	public static Comparator<Person> byName = new Comparator<Person>() {

		@Override
		public int compare(Person o1, Person o2) {
			return o1.getName().compareTo(o2.getName());
		}
		
	};
	public static Comparator<Person> byAge = new Comparator<Person>() {

		@Override
		public int compare(Person o1, Person o2) {
			return o1.getAge() - o2.getAge();
		}
		
	};
	public static Comparator<Person> byWeight = new Comparator<Person>() {

		@Override
		public int compare(Person o1, Person o2) {
			if(o1.getWeight() < o2.getWeight()) return -1;
			if(o1.getWeight() < o2.getWeight()) return 1;
			return 0;
		}
		
	};
	public static Comparator<Person> byHeight = new Comparator<Person>() {

		@Override
		public int compare(Person o1, Person o2) {
			if(o1.getHeight() < o2.getHeight()) return -1;
			if(o1.getHeight() < o2.getHeight()) return 1;
			return 0;
		}
		
	};
	
	public static Comparator<Person> byGender = new Comparator<Person>() {

		@Override
		public int compare(Person o1, Person o2) {
			return o1.getGender().compareTo(o2.getGender());
		}
		
	};
	
	public static Comparator<Person> byCountry = new Comparator<Person>() {

		@Override
		public int compare(Person p1, Person p2) {
			if (p1 instanceof Foreigner) {
				if (p2 instanceof Foreigner) { // p1 & p2 Foreigner
					return ((Foreigner)p1).getCountry().compareTo(((Foreigner)p2).getCountry());
				}
				else { // p1 Foreigner & p2 Person
					return 1;
				}
			}
			else { // p1 Person
				if (p2 instanceof Foreigner) { // p1 Person & p2 Foreigner
					return -1;
				}
				else { // p1 & p2 Person
					return 0;
				}
			}
			
		}
		
	};
}
